function c=sphfun(azel,nmax)
% sphfun.m :
% compute coefficients of the following spherical harmonic function at azimath/
% elevation angle of azel=[az el] (rad). nmax=[n m] are degree and order of
% function. output is serialized coefficients [A00 A10 A11 B11 A20 A21 B21 ...]
%
% f(a,z)=sum(sum(Pnm(cos(2*z))*(Anm*cos(m*a)+Bnm*sin(m*a))))
%    (Pnm: normalized legendre function, a:azimath angle, z:zenith angle)
%
%            Copyright(c) 2007-2008 by T.Takasu, all rights reserved
%
% version: $Revision: 1.3 $ $Date: 2008/06/03 01:26:51 $
% history: 2008/06/02 1.0 new

c=[];
p=nlegendre(cos(pi-2*azel(2)),nmax);
for n=0:nmax(1)
    c=[c,p(n+1,1)];
    for m=1:min(n,nmax(2))
        c=[c,p(n+1,m+1)*[cos(m*azel(1)),sin(m*azel(1))]];
    end
end

% normalized legendre function : p(n+1,m+1)=Pnm --------------------------------
function p=nlegendre(x,nmax)
if nmax(1)<=0, p=1; return; end
p=zeros(nmax(1)+1,nmax(2)+1); p(1:2,1)=[1;x]; 
for n=2:nmax(1), p(n+1,1)=((2*n-1)*x*p(n,1)-(n-1)*p(n-1,1))/n; end
for m=1:nmax(2)
    p(m+1,m+1)=(2*m-1)*sqrt(1-x*x)*p(m,m);
    for n=m+1:nmax(1)
        p(n+1,m+1)=((2*n-1)*x*p(n,m+1)-(n+m-1)*p(n-1,m+1))/(n-m);
    end
end
p=ncoef(nmax).*p;

% normalized coefficients ------------------------------------------------------
function c=ncoef(nmax)
persistent cn nm
if ~isempty(cn)&all(nm==nmax), c=cn; return; end
c=ones(nmax(1)+1,nmax(2)+1);
for n=1:nmax(1)
    c(n+1,1)=sqrt(2*n+1);
    for m=1:min(n,nmax(2))
        c(n+1,m+1)=sqrt(2*(2*n+1)*factorial(n-m)/factorial(n+m));
    end
end
nm=nmax; cn=c;
