function varargout=uifunc(varargin)
% uifunc.m :
% common ui functions
%
%            Copyright(c) 2007-2008 by T.Takasu, all rights reserved
%
% version: $Revision: 1.2 $ $Date: 2008/06/03 12:53:41 $
% history: 2008/06/02 1.0 new

if nargout==0, feval(varargin{:});
else [varargout{1:nargout}]=feval(varargin{:}); end

% new graph figure -------------------------------------------------------------
function h=newfig(title,varargin)
h=figure('name',title,'numbertitle','off','toolbar','none','color','w',varargin{:});

% new modal dialog -------------------------------------------------------------
function h=newdlg(title,siz,varargin)
pos=get(gcf,'position');
h=dialog('name',title,'position',[pos(1:2)+pos(3:4)/2-siz/2,siz],varargin{:});

% new ui objects ---------------------------------------------------------------
function h=newui(style,pos,str,varargin)
h=uicontrol('style',style,'position',pos,'string',str,varargin{:});
set(h,'fontname','Arial');

% new text ---------------------------------------------------------------------
function h=newtxt(pos,str,varargin)
h=newui('text',pos,str,'horizontal','left',varargin{:});

% new texts --------------------------------------------------------------------
function h=newtxts(pos,strs,varargin)
for n=1:length(strs), h(n)=newtxt(pos-[0,pos(4)*n,0,0],strs{n},varargin{:}); end

% new edit ---------------------------------------------------------------------
function h=newedt(pos,str,varargin)
h=newui('edit',pos,str,'horizontal','right','backgroundcolor','w',varargin{:});

% new popupmenu ----------------------------------------------------------------
function h=newpop(pos,strs,val,varargin)
h=newui('popupmenu',pos,strs,'max',1,'value',val,'backgroundcolor','w',varargin{:});

% new button -------------------------------------------------------------------
function h=newbtn(pos,str,cb,varargin)
h=newui('pushbutton',pos,str,'callback',[mfilename,' ',cb],varargin{:});

% new color select -------------------------------------------------------------
function h=newcolorsel(pos,color)
h=uicontrol('style','frame','position',pos-[0,0,18,0]);
set(h,'backgroundcolor',color);
set(newbtn([pos(1)+pos(3)-15,pos(2),16,pos(4)],'...',[mfilename,' cb_selcolor']),...
    'userdata',h);

function cb_selcolor
h=get(gcbo,'userdata');
set(h,'backgroundcolor',uisetcolor(get(h,'backgroundcolor'),'Select Color'));

% new font select --------------------------------------------------------------
function h=newfontsel(pos,font)
h=newtxt(pos-[0,3,18,0],'');
setfont(h,font);
newbtn([pos(1)+pos(3)-15,pos(2),16,pos(4)],'...',[mfilename,' cb_selfont'],...
       'userdata',h);

function cb_selfont
h=get(gcbo,'userdata');
font=uisetfont(get(h,'userdata'),'Select Font');
if isstruct(font), setfont(h,font); end

function setfont(h,font)
set(h,'fontname',font.FontName,'fontsize',font.FontSize,'fontangle',...
    font.FontAngle,'fontweight',font.FontWeight,'horizontal','right',...
    'userdata',font,'string',sprintf('%s %dpt',font.FontName,font.FontSize));

% new menu ---------------------------------------------------------------------
function h=newmenu(tag,top,labels,cb,chk)
delete(findall(gcf,'tag',tag));
h=uimenu(gcf,'tag',tag,'label',top,'handlevisibility','off'); m=1;
for n=find(~strcmp(labels,'-'))
    menu(n)=uimenu(h,'label',labels{n},'userdata',m,'callback',cb); m=m+1;
end
for n=find(strcmp(labels,'-'))
    if n<length(menu), set(menu(n+1),'separator','on'); end
end
if 0<chk&chk<=length(menu), set(menu(chk),'checked','on'); end

% uncheck all menus and check menu ---------------------------------------------
function chkmenu(h)
set(get(get(h,'parent'),'children'),'checked','off');
set(h,'checked','on');

% new ok/cancel buttons --------------------------------------------------------
function h=newokcancel(pos)
w=pos(3)/2;
h(1)=newbtn([pos(1:2),w,pos(4)],' OK ',[mfilename,' cb_ok']);
h(2)=newbtn([pos(1:2)+[w,0],w,pos(4)],' Cancel ',[mfilename,' cb_cancel']);

function cb_ok, set(gcf,'userdata',1);
function cb_cancel, set(gcf,'userdata',0);

% wait ok/cancel pushed --------------------------------------------------------
function ok=waitok
waitfor(gcf,'userdata');
ok=get(gcf,'userdata');

% read file dialog -------------------------------------------------------------
function files=readdlg(filt,title)
persistent ppath
wd=cd; if ~isempty(ppath), cd(ppath); end
files={}; v=version;
if str2num(v(1))>=7 % multi-select dialog if matlab7 or higher
    [file,path]=uigetfile(filt,title,'multiselect','on');
else
    [file,path]=uigetfile(filt,title);
end
cd(wd);
if isstr(file)
    files={[path,file]};
elseif iscell(file)
    for n=1:length(file), files={files{:},[path,file{n}]}; end
else
    return;
end
ppath=path;

% add move/zoom function to axes -----------------------------------------------
function moveax(ax,opt)
if nargin<1, ax=gca; end
if nargin<2, opt=[1 1 1 1]; end
p.mov=0; p.pnt=[0,0]; p.xl=[0,1]; p.yl=[0,1]; p.opt=opt;
set(ax,'userdata',p);
set(get(ax,'parent'),'windowbuttondownfcn',[mfilename,' cb_btndown'],...
    'windowbuttonmotionfcn',[mfilename,' cb_btndrag'],...
    'windowbuttonupfcn',[mfilename,' cb_btnup']);

function cb_btndown
for ax=findobj(gcf,'type','axes')'
    p=get(ax,'userdata');
    if ~isstruct(p)|~isfield(p,'mov'), continue; end
    siz=get(gcf,'position');
    pos=get(ax,'position');
    pos=[siz(3:4).*pos(1:2),siz(3:4).*(pos(1:2)+pos(3:4))];
    p.pnt=get(gcf,'currentpoint');
    if     abs(p.pnt(2)-pos(2))<10&pos(1)<p.pnt(1)&p.pnt(1)<pos(3), p.mov=1;
    elseif abs(p.pnt(1)-pos(1))<10&pos(2)<p.pnt(2)&p.pnt(2)<pos(4), p.mov=2;
    else continue; end
    if strcmp(get(gcf,'selectiontype'),'alt'), p.mov=p.mov+2; end % right button
    if ~p.opt(p.mov), continue; end
    p.xl=get(ax,'xlim'); p.yl=get(ax,'ylim');
    set(ax,'userdata',p);
    cur={'right','top'};
    if p.mov<=2, set(gcf,'pointer',cur{p.mov}); else setcurzoom(p.mov==3); end
end
function cb_btndrag
for ax=findobj(gcf,'type','axes')'
    p=get(ax,'userdata');
    if ~isstruct(p)|~isfield(p,'mov')|p.mov<=0, continue; end
    siz=get(gcf,'position');
    pos=get(ax,'position');
    dp=(get(gcf,'currentpoint')-p.pnt)./siz(3:4)./pos(3:4);
    switch p.mov
    case 1, p.xl=p.xl-(p.xl(2)-p.xl(1))*dp(1); % move x axis
    case 2, p.yl=p.yl-(p.yl(2)-p.yl(1))*dp(2); % move y axis
    case 3, p.xl=p.xl(1)+(p.xl(2)-p.xl(1))*(0.5+[-0.5,0.5]*0.1^dp(1)); % zoom x axis
    case 4, p.yl=p.yl(1)+(p.yl(2)-p.yl(1))*(0.5+[-0.5,0.5]*0.1^dp(2)); % zoom y axis
    end
    set(ax,'xlim',p.xl,'ylim',p.yl);
end

function cb_btnup
for ax=findobj(gcf,'type','axes')'
    p=get(ax,'userdata');
    if ~isstruct(p)|~isfield(p,'mov')|p.mov<=0, continue; end
    p.mov=0;
    set(ax,'userdata',p);
    set(gcf,'pointer','arrow');
end

function setcurzoom(opt)
o=nan;
p=[o,o,o,o,o,o,o,2,1,2,o,o,o,o,o,o
   o,o,o,o,o,o,2,1,1,1,2,o,o,o,o,o
   o,o,o,o,o,2,1,1,1,1,1,2,o,o,o,o
   o,o,o,o,2,1,1,1,1,1,1,1,2,o,o,o
   o,o,o,2,1,1,1,1,1,1,1,1,1,2,o,o
   o,o,o,2,2,2,2,2,1,2,2,2,2,2,o,o
   o,o,o,o,o,o,o,2,1,2,o,o,o,o,o,o
   o,o,o,o,o,o,o,2,1,2,o,o,o,o,o,o
   o,o,o,o,o,2,2,2,1,2,2,2,o,o,o,o
   o,o,o,o,o,2,1,1,1,1,1,2,o,o,o,o
   o,o,o,o,o,2,2,2,1,2,2,2,o,o,o,o
   o,o,o,o,o,o,o,2,1,2,o,o,o,o,o,o
   o,o,o,o,o,2,2,2,1,2,2,2,o,o,o,o
   o,o,o,o,o,2,1,1,1,1,1,2,o,o,o,o
   o,o,o,o,o,o,2,1,1,1,2,o,o,o,o,o
   o,o,o,o,o,o,o,2,1,2,o,o,o,o,o,o];
if opt, p=fliplr(p'); end
set(gcf,'pointer','custom','pointershapecdata',p,'pointershapehotspot',[9,9])
